@@ -2,6 +2,7 @@
 #include <QFile>
 #include <QFileInfo>
 #include "common/util.h"
+#include "map/crs.h"
 #include "gpxparser.h"
 #include "tcxparser.h"
 #include "csvparser.h"
@@ -154,6 +155,70 @@ Data::Data(const QString &fileName, bool tryUnknown)
 	}
 }
 
+Data::Data(const QUrl &url)
+{
+	bool caOk, cbOk, ccOk;
+	Projection proj(GCS::WGS84());
+
+	_valid = false;
+
+	QStringList parts(url.path().split(';'));
+	if (parts.size() < 1) {
+		_errorString = "Syntax error";
+		return;
+	}
+	QStringList coords(parts.at(0).split(','));
+	if (coords.size() < 2 || coords.size() > 3) {
+		_errorString = "Syntax error";
+		return;
+	}
+	double ca = coords.at(0).toDouble(&caOk);
+	double cb = coords.at(1).toDouble(&cbOk);
+	double cc = NAN;
+	if (!(caOk && cbOk)) {
+		_errorString = "Invalid coordinates";
+		return;
+	}
+	if (coords.size() > 2) {
+		cc = coords.at(2).toDouble(&ccOk);
+		if (!ccOk) {
+			_errorString = "Invalid elevation";
+			return;
+		}
+	}
+
+	if (parts.size() > 1) {
+		QStringList crsp(parts.at(1).split('='));
+		if (crsp.size() != 2) {
+			_errorString = "Syntax error";
+			return;
+		}
+		if (!crsp.at(0).compare("crs", Qt::CaseInsensitive)) {
+			if (crsp.at(1).compare("wgs84", Qt::CaseInsensitive)) {
+				proj = CRS::projection(crsp.at(1));
+				if (!proj.isValid()) {
+					_errorString = "Unknown CRS";
+					return;
+				}
+			}
+		}
+	}
+
+	CoordinateSystem::AxisOrder ao = proj.coordinateSystem().axisOrder();
+	PointD p(ao == CoordinateSystem::XY ? PointD(ca, cb) : PointD(cb, ca));
+	Coordinates c(proj.xy2ll(p));
+	if (!c.isValid()) {
+		_errorString = "Invalid coordinates";
+		return;
+	}
+
+	Waypoint w(c);
+	w.setElevation(cc);
+	_waypoints.append(w);
+
+	_valid = true;
+}
+
 QString Data::formats()
 {
 	return
