@@ -173,19 +173,41 @@ static bool polygonCb(const MapData::Poly *polygon, void *context)
 static bool polygonPointCb(const MapData::Poly *polygon, void *context)
 {
 	QList<MapData::Point> *points = (QList<MapData::Point>*)context;
-	uint baseType = polygon->type()>>16;
+	uint type = polygon->type();
+	uint baseType = type>>16;
 
-	if (!polygon->label().isEmpty() || baseType == TSSLPT || baseType == RCTLPT
-	  || baseType == I_TRNBSN
-	  || polygon->type() == SUBTYPE(ACHARE, 2)
-	  || polygon->type() == SUBTYPE(ACHARE, 3)
-	  || polygon->type() == SUBTYPE(ACHARE, 9)
-	  || polygon->type() == SUBTYPE(I_ACHARE, 2)
-	  || polygon->type() == SUBTYPE(I_ACHARE, 3)
-	  || polygon->type() == SUBTYPE(I_ACHARE, 9)
-	  || polygon->type() == SUBTYPE(I_BERTHS, 6))
-		points->append(MapData::Point(polygon->type(), polygon->bounds().center(),
-		  polygon->label(), polygon->param()));
+	if (baseType == TSSLPT || baseType == RCTLPT || baseType == I_TRNBSN
+	  || baseType == BRIDGE || baseType == I_BRIDGE || baseType == BUAARE
+	  || baseType == LNDARE || baseType == LNDRGN
+	  || type == SUBTYPE(ACHARE, 2) || type == SUBTYPE(I_ACHARE, 2)
+	  || type == SUBTYPE(ACHARE, 3) || type == SUBTYPE(I_ACHARE, 3)
+	  || type == SUBTYPE(ACHARE, 9) || type == SUBTYPE(I_ACHARE, 9)
+	  || type == SUBTYPE(I_BERTHS, 6)
+	  || type == SUBTYPE(RESARE, 1) || type == SUBTYPE(I_RESARE, 1)
+	  || type == SUBTYPE(RESARE, 2) || type == SUBTYPE(I_RESARE, 2)
+	  || type == SUBTYPE(RESARE, 4) || type == SUBTYPE(I_RESARE, 4)
+	  || type == SUBTYPE(RESARE, 5) || type == SUBTYPE(I_RESARE, 5)
+	  || type == SUBTYPE(RESARE, 6) || type == SUBTYPE(I_RESARE, 6)
+	  || type == SUBTYPE(RESARE, 7) || type == SUBTYPE(I_RESARE, 7)
+	  || type == SUBTYPE(RESARE, 9) || type == SUBTYPE(I_RESARE, 9)
+	  || type == SUBTYPE(RESARE, 12) || type == SUBTYPE(I_RESARE, 12)
+	  || type == SUBTYPE(RESARE, 17) || type == SUBTYPE(I_RESARE, 17)
+	  || type == SUBTYPE(RESARE, 22) || type == SUBTYPE(I_RESARE, 22)
+	  || type == SUBTYPE(RESARE, 23) || type == SUBTYPE(I_RESARE, 23))
+		points->append(MapData::Point(baseType, polygon->bounds().center(),
+		  polygon->attributes(), polygon->HUNI(), true));
+
+	return true;
+}
+
+static bool linePointCb(const MapData::Line *line, void *context)
+{
+	QList<MapData::Point> *points = (QList<MapData::Point>*)context;
+	uint baseType = line->type()>>16;
+
+	if (baseType == RDOCAL || baseType == I_RDOCAL)
+		points->append(MapData::Point(baseType, line->bounds().center(),
+		  line->attributes(), 1));
 
 	return true;
 }
@@ -207,10 +229,8 @@ static Coordinates point(const ISO8211::Record &r, uint COMF)
 	return coordinates(x, y, COMF);
 }
 
-static uint depthLevel(const QByteArray &ba)
+static uint depthLevel(double minDepth)
 {
-	double minDepth = ba.isEmpty() ? -1 : ba.toDouble();
-
 	if (minDepth < 0)
 		return 0;
 	else if (minDepth < 2)
@@ -275,72 +295,171 @@ static QString weed(uint type)
 	}
 }
 
-MapData::Point::Point(uint type, const Coordinates &c, const QString &label,
-  const QVector<QByteArray> &params) : _type(type), _pos(c), _label(label)
+static uint restrictionCategory(uint type, const MapData::Attributes &attr)
 {
-	_id = ((quint64)order(type))<<32 | (uint)qHash(c);
+	uint catrea = attr.value(CATREA).toUInt();
 
-	if (type>>16 == I_DISMAR && params.size()) {
-		_label = hUnits((type>>8)&0xFF) + " " + QString::fromLatin1(params.at(0));
-		_type = SUBTYPE(I_DISMAR, type & 0xFF);
-	} else if ((type>>16 == I_RDOCAL || type>>16 == RDOCAL) && params.size() > 1) {
-		if (!params.at(1).isEmpty())
-			_label = QString("VHF ") + QString::fromLatin1(params.at(1));
-		_param = QVariant(params.at(0).toDouble());
-	} else if (type>>16 == CURENT && params.size() > 1) {
-		if (!params.at(1).isEmpty())
-			_label = QString::fromLatin1(params.at(1))
-			  + QString::fromUtf8("\xE2\x80\x89kt");
-		_param = QVariant(params.at(0).toDouble());
-	} else if (type>>16 == I_SISTAT || type>>16 == SISTAT) {
+	if (!catrea) {
+		uint restrn = attr.value(
+		  (type == RESARE) ? RESTRN : I_RESTRN).toUInt();
+
+		if (restrn == 1)
+			return 2;
+		else if (restrn == 7)
+			return 17;
+		else
+			return 0;
+	} else
+		return catrea;
+}
+
+MapData::Point::Point(uint type, const Coordinates &c, const QString &label)
+  : _type(SUBTYPE(type, 0)), _pos(c), _label(label), _polygon(false)
+{
+	_id = ((quint64)order(_type))<<32 | (uint)qHash(c);
+}
+
+MapData::Point::Point(uint type, const Coordinates &c, const Attributes &attr,
+  uint HUNI, bool polygon) : _pos(c), _attr(attr), _polygon(polygon)
+{
+	uint subtype = 0;
+
+	if (type == HRBFAC)
+		subtype = CATHAF;
+	else if (type == I_HRBFAC)
+		subtype = I_CATHAF;
+	else if (type == LNDMRK)
+		subtype = CATLMK;
+	else if (type == WRECKS)
+		subtype = CATWRK;
+	else if (type == MORFAC)
+		subtype = CATMOR;
+	else if (type == UWTROC)
+		subtype = WATLEV;
+	else if (type == BUAARE)
+		subtype = CATBUA;
+	else if (type == SMCFAC)
+		subtype = CATSCF;
+	else if (type == BUISGL)
+		subtype = FUNCTN;
+	else if (type == WATTUR)
+		subtype = CATWAT;
+	else if (type == RDOCAL)
+		subtype = TRAFIC;
+	else if (type == I_RDOCAL)
+		subtype = TRAFIC;
+	else if (type == SILTNK)
+		subtype = CATSIL;
+	else if (type == WEDKLP)
+		subtype = CATWED;
+	else if (type == LIGHTS)
+		subtype = CATLIT;
+	else if (type == I_DISMAR)
+		subtype = CATDIS;
+	else if (type == I_BERTHS)
+		subtype = I_CATBRT;
+	else if (type == ACHARE)
+		subtype = CATACH;
+	else if (type == I_ACHARE)
+		subtype = I_CATACH;
+
+	QList<QByteArray> list(_attr.value(subtype).split(','));
+	std::sort(list.begin(), list.end());
+	_type = (type == RESARE || type == I_RESARE)
+	  ? SUBTYPE(type, restrictionCategory(type, _attr))
+	  : SUBTYPE(type, list.first().toUInt());
+	_id = ((quint64)order(_type))<<32 | (uint)qHash(c);
+	_label = QString::fromLatin1(_attr.value(OBJNAM));
+
+	if (type == I_DISMAR) {
+		if (_attr.contains(I_WTWDIS) && _attr.contains(I_HUNITS))
+			_label = hUnits(_attr.value(I_HUNITS).toUInt()) + " "
+			  + QString::fromLatin1(_attr.value(I_WTWDIS));
+	} else if (type == I_RDOCAL || type == RDOCAL) {
+		QByteArray cc(_attr.value(COMCHA));
+		if (!cc.isEmpty())
+			_label = QString("VHF ") + QString::fromLatin1(cc);
+	} else if (type == CURENT) {
+		QByteArray cv(_attr.value(CURVEL));
+		if (!cv.isEmpty())
+			_label = QString::fromLatin1(cv) + QString::fromUtf8("\xE2\x80\x89kt");
+	} else if (type == SISTAT) {
+		if (_label.isEmpty() && _attr.contains(CATSIT))
+			_label = sistat(_attr.value(CATSIT).toUInt());
+	} else if (type == I_SISTAT) {
+		if (_label.isEmpty() && _attr.contains(I_CATSIT))
+			_label = sistat(_attr.value(I_CATSIT).toUInt());
+	} else if (type == WEDKLP) {
 		if (_label.isEmpty())
-			_label = sistat(type & 0xFF);
-		_type = TYPE(SISTAT);
-	} else if (type>>16 == WEDKLP) {
+			_label = weed(_type & 0xFF);
+	} else if (type == LNDELV) {
 		if (_label.isEmpty())
-			_label = weed(type & 0xFF);
-	} else if (type>>16 == LNDELV && params.size()) {
-		if (_label.isEmpty())
-			_label = QString::fromLatin1(params.at(0))
+			_label = QString::fromLatin1(_attr.value(ELEVAT))
 			  + QString::fromUtf8("\xE2\x80\x89m");
 		else
-			_label += "\n(" + QString::fromLatin1(params.at(0))
+			_label += "\n(" + QString::fromLatin1(_attr.value(ELEVAT))
 			  + "\xE2\x80\x89m)";
-	} else if ((type == TYPE(TSSLPT) || type == TYPE(RCTLPT)) && params.size())
-		_param = QVariant(params.at(0).toDouble());
-}
-
-MapData::Point::Point(uint type, const Coordinates &c, const QString &label,
-  const QVariant &param) : _type(type), _pos(c), _label(label), _param(param)
-{
-	_id = ((quint64)order(type))<<32 | (uint)qHash(c);
-}
-
-MapData::Poly::Poly(uint type, const Polygon &path, const QString &label,
-  const QVector<QByteArray> &params, uint HUNI) : _type(type), _path(path)
-{
-	if (type == TYPE(DEPARE) && params.size())
-		_type = SUBTYPE(DEPARE, depthLevel(params.at(0)));
-	else if ((type == TYPE(TSSLPT) || type == TYPE(RCTLPT)) && params.size())
-		_param = QVariant(params.at(0).toDouble());
-	else if ((type == TYPE(BRIDGE) || type == TYPE(I_BRIDGE))
-	  && params.size()) {
-		double clr = params.at(0).toDouble();
+	} else if (type == BRIDGE || type == I_BRIDGE) {
+		double clr = _attr.value(VERCLR).toDouble();
 		if (clr > 0) {
 			_label = QString::fromUtf8("\xE2\x86\x95") + UNIT_SPACE
 			  + QString::number(clr) + UNIT_SPACE + hUnits(HUNI);
 		}
-	} else if (type>>16 == LNDARE || type>>16 == SEAARE || type>>16 == BERTHS
-	  || type>>16 == I_BERTHS || type>>16 == BUAARE)
-		_label = label;
+	}
+}
+
+MapData::Poly::Poly(uint type, const Polygon &path, const Attributes &attr,
+  uint HUNI) : _path(path), _attr(attr), _HUNI(HUNI)
+{
+	uint subtype = 0;
+
+	if (type == ACHARE)
+		subtype = CATACH;
+	else if (type == I_ACHARE)
+		subtype = I_CATACH;
+	else if (type == HRBFAC)
+		subtype = CATHAF;
+	else if (type == MARKUL)
+		subtype = CATMFA;
+	else if (type == I_BERTHS)
+		subtype = I_CATBRT;
+
+	switch (type) {
+		case DEPARE:
+			_type = SUBTYPE(type, depthLevel(_attr.value(DRVAL1).toDouble()));
+			break;
+		case RESARE:
+		case I_RESARE:
+			_type = SUBTYPE(type, restrictionCategory(type, attr));
+			break;
+		default:
+			_type = SUBTYPE(type, _attr.value(subtype).toUInt());
+	}
+
 }
 
 MapData::Line::Line(uint type, const QVector<Coordinates> &path,
-  const QString &label, const QVector<QByteArray> &params)
-  : _type(type), _path(path), _label(label)
+  const Attributes &attr) : _path(path), _attr(attr)
 {
-	if ((type == TYPE(DEPCNT) || type == TYPE(LNDELV)) && params.size())
-		_label = QString::fromLatin1(params.at(0));
+	uint subtype = 0;
+
+	if (type == RECTRC)
+		subtype = CATTRK;
+	else if (type == RCRTCL)
+		subtype = CATTRK;
+	else if (type == RDOCAL)
+		subtype = TRAFIC;
+	else if (type == I_RDOCAL)
+		subtype = TRAFIC;
+
+	_type = SUBTYPE(type, _attr.value(subtype).toUInt());
+
+	if (type == DEPCNT)
+		_label = QString::fromLatin1(_attr.value(VALDCO));
+	else if (type == LNDELV)
+		_label = QString::fromLatin1(_attr.value(ELEVAT));
+	else
+		_label = QString::fromLatin1(_attr.value(OBJNAM));
 }
 
 RectC MapData::Line::bounds() const
@@ -581,169 +700,46 @@ Polygon MapData::polyGeometry(const ISO8211::Record &r, const RecordMap &vc,
 	return path;
 }
 
-MapData::Attr MapData::pointAttr(const ISO8211::Record &r, uint OBJL)
+MapData::Attributes MapData::attributes(const ISO8211::Record &r)
 {
-	QString label;
-	QVector<QByteArray> params(2);
-	uint subtype = 0;
+	Attributes attr;
 
 	const ISO8211::Field *ATTF = ISO8211::field(r, "ATTF");
 	if (!(ATTF && ATTF->data().at(0).size() == 2))
-		return Attr();
+		return attr;
 
 	for (int i = 0; i < ATTF->data().size(); i++) {
 		const QVector<QVariant> &av = ATTF->data().at(i);
-		uint key = av.at(0).toUInt();
-
-		if (key == OBJNAM)
-			label = QString::fromLatin1(av.at(1).toByteArray());
-
-		if ((OBJL == HRBFAC && key == CATHAF)
-		  || (OBJL == I_HRBFAC && key == I_CATHAF)
-		  || (OBJL == LNDMRK && key == CATLMK)
-		  || (OBJL == WRECKS && key == CATWRK)
-		  || (OBJL == MORFAC && key == CATMOR)
-		  || (OBJL == UWTROC && key == WATLEV)
-		  || (OBJL == BUAARE && key == CATBUA)
-		  || (OBJL == SMCFAC && key == CATSCF)
-		  || (OBJL == BUISGL && key == FUNCTN)
-		  || (OBJL == WATTUR && key == CATWAT)
-		  || (OBJL == SISTAT && key == CATSIT)
-		  || (OBJL == I_SISTAT && key == I_CATSIT)
-		  || (OBJL == RDOCAL && key == TRAFIC)
-		  || (OBJL == I_RDOCAL && key == TRAFIC)
-		  || (OBJL == SILTNK && key == CATSIL)
-		  || (OBJL == WEDKLP && key == CATWED))
-			subtype = av.at(1).toByteArray().toUInt();
-		else if (OBJL == I_DISMAR && key == CATDIS)
-			subtype |= av.at(1).toByteArray().toUInt();
-		else if (OBJL == I_DISMAR && key == I_HUNITS)
-			subtype |= av.at(1).toByteArray().toUInt() << 8;
-
-		if ((OBJL == I_DISMAR && key == I_WTWDIS)
-		  || (OBJL == RDOCAL && key == ORIENT)
-		  || (OBJL == I_RDOCAL && key == ORIENT)
-		  || (OBJL == CURENT && key == ORIENT)
-		  || (OBJL == LNDELV && key == ELEVAT)
-		  || (OBJL == TSSLPT && key == ORIENT)
-		  || (OBJL == RCTLPT && key == ORIENT))
-			params[0] = av.at(1).toByteArray();
-		if ((OBJL == I_RDOCAL && key == COMCHA)
-		  || (OBJL == RDOCAL && key == COMCHA)
-		  || (OBJL == CURENT && key == CURVEL))
-			params[1] = av.at(1).toByteArray();
+		attr.insert(av.at(0).toUInt(), av.at(1).toByteArray());
 	}
 
-	return Attr(subtype, label, params);
-}
-
-MapData::Attr MapData::lineAttr(const ISO8211::Record &r, uint OBJL)
-{
-	QString label;
-	QVector<QByteArray> params(1);
-	uint subtype = 0;
-
-	const ISO8211::Field *ATTF = ISO8211::field(r, "ATTF");
-	if (!(ATTF && ATTF->data().at(0).size() == 2))
-		return Attr();
-
-	for (int i = 0; i < ATTF->data().size(); i++) {
-		const QVector<QVariant> &av = ATTF->data().at(i);
-		uint key = av.at(0).toUInt();
-
-		if (key == OBJNAM)
-			label = QString::fromLatin1(av.at(1).toByteArray());
-
-		if ((OBJL == RECTRC || OBJL == RCRTCL) && key == CATTRK)
-			subtype = av.at(1).toByteArray().toUInt();
-
-		if ((OBJL == DEPCNT && key == VALDCO)
-		  || (OBJL == LNDELV && key == ELEVAT))
-			params[0] = av.at(1).toByteArray();
-	}
-
-	return Attr(subtype, label, params);
-}
-
-MapData::Attr MapData::polyAttr(const ISO8211::Record &r, uint OBJL)
-{
-	QString label;
-	QVector<QByteArray> params(1);
-	uint subtype = 0;
-
-	const ISO8211::Field *ATTF = ISO8211::field(r, "ATTF");
-	if (!(ATTF && ATTF->data().at(0).size() == 2))
-		return Attr();
-
-	for (int i = 0; i < ATTF->data().size(); i++) {
-		const QVector<QVariant> &av = ATTF->data().at(i);
-		uint key = av.at(0).toUInt();
-
-		if (key == OBJNAM)
-			label = QString::fromLatin1(av.at(1).toByteArray());
-
-		if ((OBJL == RESARE && key == CATREA)
-		  || (OBJL == I_RESARE && key == CATREA)
-		  || (OBJL == ACHARE && key == CATACH)
-		  || (OBJL == I_ACHARE && key == I_CATACH)
-		  || (OBJL == HRBFAC && key == CATHAF)
-		  || (OBJL == MARKUL && key == CATMFA)
-		  || (OBJL == I_BERTHS && key == I_CATBRT))
-			subtype = av.at(1).toByteArray().toUInt();
-		else if ((OBJL == RESARE && key == RESTRN)
-		  || (OBJL == I_RESARE && key == I_RESTRN)) {
-			if (av.at(1).toByteArray().toUInt() == 1)
-				subtype = 2;
-			if (av.at(1).toByteArray().toUInt() == 7)
-				subtype = 17;
-		}
-
-		if ((OBJL == TSSLPT && key == ORIENT)
-		  || (OBJL == RCTLPT && key == ORIENT)
-		  || (OBJL == DEPARE && key == DRVAL1))
-			params[0] = av.at(1).toByteArray();
-		if ((OBJL == BRIDGE && key == VERCLR)
-		  || (OBJL == I_BRIDGE && key == VERCLR))
-			params[0] = av.at(1).toByteArray();
-	}
-
-	return Attr(subtype, label, params);
+	return attr;
 }
 
 MapData::Point *MapData::pointObject(const Sounding &s)
 {
-	return new Point(TYPE(SOUNDG), s.c, QString::number(s.depth),
-	  QVector<QByteArray>());
+	return new Point(SOUNDG, s.c, QString::number(s.depth));
 }
 
 MapData::Point *MapData::pointObject(const ISO8211::Record &r,
-  const RecordMap &vi, const RecordMap &vc, uint COMF, uint OBJL)
+  const RecordMap &vi, const RecordMap &vc, uint COMF, uint OBJL, uint HUNI)
 {
 	Coordinates c(pointGeometry(r, vi, vc, COMF));
-	Attr attr(pointAttr(r, OBJL));
-
-	return (c.isNull() ? 0 : new Point(SUBTYPE(OBJL,attr.subtype()), c,
-	  attr.label(), attr.params()));
+	return (c.isNull() ? 0 : new Point(OBJL, c, attributes(r), HUNI));
 }
 
 MapData::Line *MapData::lineObject(const ISO8211::Record &r,
   const RecordMap &vc, const RecordMap &ve, uint COMF, uint OBJL)
 {
 	QVector<Coordinates> path(lineGeometry(r, vc, ve, COMF));
-	Attr attr(lineAttr(r, OBJL));
-
-	return (path.isEmpty() ? 0 : new Line(SUBTYPE(OBJL, attr.subtype()), path,
-	  attr.label(), attr.params()));
+	return (path.isEmpty() ? 0 : new Line(OBJL, path, attributes(r)));
 }
 
 MapData::Poly *MapData::polyObject(const ISO8211::Record &r,
   const RecordMap &vc, const RecordMap &ve, uint COMF, uint OBJL, uint HUNI)
 {
 	Polygon path(polyGeometry(r, vc, ve, COMF));
-	Attr attr(polyAttr(r, OBJL));
-
-	return (path.isEmpty() ? 0 : new Poly(SUBTYPE(OBJL, attr.subtype()), path,
-	  attr.label(), attr.params(), HUNI));
+	return (path.isEmpty() ? 0 : new Poly(OBJL, path, attributes(r), HUNI));
 }
 
 bool MapData::processRecord(const ISO8211::Record &record,
@@ -828,7 +824,7 @@ MapData::MapData(const QString &path)
 						_points.Insert(min, max, point);
 					}
 				} else {
-					if ((point = pointObject(r, vi, vc, COMF, OBJL))) {
+					if ((point = pointObject(r, vi, vc, COMF, OBJL, HUNI))) {
 						pointBounds(point->pos(), min, max);
 						_points.Insert(min, max, point);
 					} else
@@ -875,6 +871,7 @@ void MapData::points(const RectC &rect, QList<Point> *points) const
 	rectcBounds(rect, min, max);
 	_points.Search(min, max, pointCb, points);
 	_areas.Search(min, max, polygonPointCb, points);
+	_lines.Search(min, max, linePointCb, points);
 }
 
 void MapData::lines(const RectC &rect, QList<Line> *lines) const
