@@ -87,6 +87,10 @@ static Projection parseProjection(const QString &str, const GCS &gcs)
 			return Projection(PCS(gcs, Conversion(9809, Conversion::Setup(
 			  52.15616055555555, 5.38763888888889, 0.9999079, 155000, 463000,
 			  NAN, NAN), 9001)));
+		case 125: // Israeli New grid
+			return Projection(PCS(gcs, Conversion(9807, Conversion::Setup(
+			  31.7343936111111, 35.2045169444444, 1.0000067, 219529.584,
+			  626907.39, NAN, NAN), 9001)));
 		case 184: // Swedish Grid
 			return Projection(PCS(gcs, Conversion(9807, Conversion::Setup(0,
 			  15.808278, 1, 1500000, 0, NAN, NAN), 9001)));
@@ -361,7 +365,9 @@ void RMap::load(const Projection &in, const Projection &out, qreal deviceRatio,
 	Q_UNUSED(out);
 
 	_mapRatio = hidpi ? deviceRatio : 1.0;
-	_file.open(QIODevice::ReadOnly);
+	if (!_file.open(QIODevice::ReadOnly))
+		qWarning("%s: %s", qUtf8Printable(_file.fileName()),
+		  qUtf8Printable(_file.errorString()));
 }
 
 void RMap::unload()
@@ -449,7 +455,7 @@ void RMap::draw(QPainter *painter, const QRectF &rect, Flags flags)
 			}
 
 			if (pixmap.isNull())
-				qWarning("%s: error loading tile image", qPrintable(key));
+				qWarning("%s: error loading tile image", qUtf8Printable(key));
 			else {
 				pixmap.setDevicePixelRatio(_mapRatio);
 				QPointF tp(tl.x() + i * ts.width(), tl.y() + j * ts.height());
